@@ -16,7 +16,7 @@ from django.utils.translation import ugettext as _
 
 from feincms import settings
 
-FRONTEND_EDITING_MATCHER = re.compile(r'(\d+)/(\w+)/(\d+)')
+FRONTEND_EDITING_MATCHER = re.compile(r'(\d+)\|(\w+)\|(\d+)')
 
 
 class ItemEditorForm(forms.ModelForm):
@@ -33,6 +33,17 @@ class ItemEditor(admin.ModelAdmin):
         This does not need to (and should not) include ``template``
     """
 
+    def __init__(self, *args, **kwargs):
+        # Make sure all models are completely loaded before attempting to
+        # proceed. The dynamic nature of FeinCMS models makes this necessary.
+        # For more informations, have a look at issue #23 on github:
+        # http://github.com/matthiask/feincms/issues#issue/23
+        from django.core.management.validation import get_validation_errors
+        from StringIO import StringIO
+        get_validation_errors(StringIO(), None)
+
+        super(ItemEditor, self).__init__(*args, **kwargs)
+
     def _formfield_callback(self, request):
         if settings.DJANGO10_COMPAT:
             # This should compare for Django SVN before [9761] (From 2009-01-16),
@@ -157,6 +168,7 @@ class ItemEditor(admin.ModelAdmin):
                 formfield_callback=self._formfield_callback(request=request))
             ) for content_type in self.model._feincms_content_types]
 
+        formsets = []
         if request.method == 'POST':
             FORM_DATA = {}
             model_form = ModelForm(request.POST, request.FILES, instance=obj)
@@ -166,11 +178,31 @@ class ItemEditor(admin.ModelAdmin):
                     prefix=content_type.__name__.lower())
                 for content_type, formset_class in inline_formset_types]
 
-            if model_form.is_valid() and all_valid(inline_formsets):
-                model_form.save(commit=False)
+            if model_form.is_valid():
+                form_validated = True
+                new_object = self.save_form(request, model_form, change=True)
+            else:
+                form_validated = False
+                new_object = obj
+            prefixes = {}
+            for FormSet in self.get_formsets(request, new_object):
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
+                formset = FormSet(request.POST, request.FILES,
+                                  instance=new_object, prefix=prefix)
+                formsets.append(formset)
+
+            if all_valid(inline_formsets+formsets) and form_validated:
+                #model_form.save(commit=False)
+                model_form.save_m2m()
                 for formset in inline_formsets:
                     formset.save()
-                model_form.save(commit=True)
+                #model_form.save(commit=True)
+                self.save_model(request, new_object, model_form, change=True)
+                for formset in formsets:
+                    self.save_formset(request, model_form, formset, change=True)
 
                 msg = _('The %(name)s "%(obj)s" was changed successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
                 if request.POST.has_key("_continue"):
@@ -225,12 +257,28 @@ class ItemEditor(admin.ModelAdmin):
                 formset_class(FORM_DATA, instance=obj, prefix=content_type.__name__.lower())
                 for content_type, formset_class in inline_formset_types
             ]
+            prefixes = {}
+            for FormSet in self.get_formsets(request, obj):
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
+                formset = FormSet(instance=obj, prefix=prefix)
+                formsets.append(formset)
         else:
             model_form = ModelForm(instance=obj)
             inline_formsets = [
                 formset_class(instance=obj, prefix=content_type.__name__.lower())
                 for content_type, formset_class in inline_formset_types
             ]
+            prefixes = {}
+            for FormSet in self.get_formsets(request, obj):
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
+                formset = FormSet(instance=obj, prefix=prefix)
+                formsets.append(formset)
 
         # Prepare mapping of content types to their prettified names
         content_types = []
@@ -240,23 +288,40 @@ class ItemEditor(admin.ModelAdmin):
 
         context = {}
 
+        media = self.media + model_form.media
+
+        inline_admin_formsets = []
+        for inline, formset in zip(self.inline_instances, formsets):
+            fieldsets = list(inline.get_fieldsets(request, obj))
+            inline_admin_formset = helpers.InlineAdminFormSet(inline, formset, fieldsets)
+            inline_admin_formsets.append(inline_admin_formset)
+            media = media + inline_admin_formset.media
+
+
         if hasattr(self.model, '_feincms_templates'):
             if 'template_key' not in self.show_on_top:
                 self.show_on_top = ['template_key'] + list(self.show_on_top)
 
             context['available_templates'] = self.model._feincms_templates
 
+        if hasattr(self.model, 'parent'):
+            context['has_parent_attribute'] = True
+
         context.update({
+            'has_add_permission': self.has_add_permission(request),
+            'has_change_permission': self.has_change_permission(request, obj=obj),
+            'has_delete_permission': self.has_delete_permission(request, obj=obj),
             'change': obj.pk is not None,
             'title': _('Change %s') % force_unicode(opts.verbose_name),
             'opts': opts,
             'object': obj,
             'object_form': model_form,
             'inline_formsets': inline_formsets,
+            'inline_admin_formsets': inline_admin_formsets,
             'content_types': content_types,
             'top_fields': [model_form[field] for field in self.show_on_top],
             'settings_fields': [field for field in model_form if field.name not in self.show_on_top],
-            'media': self.media + model_form.media,
+            'media': media,
             'errors': helpers.AdminErrorList(model_form, inline_formsets),
             'FEINCMS_ADMIN_MEDIA': settings.FEINCMS_ADMIN_MEDIA,
             'FEINCMS_ADMIN_MEDIA_HOTLINKING': settings.FEINCMS_ADMIN_MEDIA_HOTLINKING,
