@@ -61,19 +61,37 @@ function move_item (region_id, item) {
 
 function poorify_rich(item){
     item.children(".item-content").hide();
-    // TODO: Change this to allow other editors:
-    if (item.find("div[id^=richtext]").length > 0) {
-        var editor_id = item.find(".mceEditor").prev().attr("id");
-        tinyMCE.execCommand('mceRemoveControl', false, editor_id);
-    }
+
+    item.find('div.item-content textarea[class^=item-richtext-]').each(function(){
+        var field = $(this);
+        var classes = field.attr('class').split(' ');
+        $.each(classes, function() {
+            if(this.match('^item-richtext-')) {
+                var remove_func = undefined;
+                try { remove_func = eval('feincms_richtext_remove_' + this.substr(14)); } catch(e) {}
+                if(typeof(remove_func) == 'function'){
+                    remove_func(field);
+                }
+            }
+        });
+    });
 }
 function richify_poor(item){
     item.children(".item-content").show();
-    // TODO: Change this to allow other editors:
-    if (item.find("div[id^=richtext]").length > 0) {
-        var editor_id = item.find('textarea[name*=richtext]').attr("id");
-        tinyMCE.execCommand('mceAddControl', false, editor_id);
-    }
+
+    item.find('div.item-content textarea[class^=item-richtext-]').each(function(){
+        var field = $(this);
+        var classes = field.attr('class').split(' ');
+        $.each(classes, function() {
+            if(this.match('^item-richtext-')) {
+                var add_func = undefined;
+                try { add_func = eval('feincms_richtext_add_' + this.substr(14)); } catch(e) {}
+                if(typeof(add_func) == 'function'){
+                    add_func(field);
+                }
+            }
+        });
+    });
 }
 
 function sort_by_ordering(e1, e2) {
@@ -190,9 +208,9 @@ $(document).ready(function(){
         return false;
     });
 
-    current_template = $('input[name=template_key][checked]').val();
+    current_template = $('input[name=template_key][checked], select[name=template_key]').val();
 
-    $('input[name=template_key]').click(function(){
+    function on_template_key_changed(){
         var input_element = this;
         var new_template = this.value;
 
@@ -236,7 +254,10 @@ $(document).ready(function(){
         });
 
         return false;
-    });
+    }
+
+    $('input[type=radio][name=template_key]').click(on_template_key_changed);
+    $('select[name=template_key]').change(on_template_key_changed);
 
     $("fieldset.order-item").live('click', function(){
         if(!$(this).hasClass('active-item')) {
